@@ -9,10 +9,12 @@
 #'
 #' @param excl Character string (or vector of character strings) denoting parameters to exclude. Used in conjunction with \code{params} argument to select parameters of interest.
 #'
-#' @param ISB Ignore Square Brackets (ISB). Logical specifying whether square brackets should be ignored in the \code{params} and \code{excl} arguments. If \code{FALSE}, square brackets are ignored - input from \code{params} and \code{excl} are otherwise matched exactly. If \code{TRUE}, square brackets are not ignored - input from \code{params} and \code{excl} are matched using grep, allowing partial names to be used when specifying parameters of interest.
+#' @param ISB Ignore Square Brackets (ISB). Logical specifying whether square brackets should be ignored in the \code{params} and \code{excl} arguments. If \code{TRUE}, square brackets are ignored - input from \code{params} and \code{excl} are otherwise matched exactly. If \code{FALSE}, square brackets are not ignored - input from \code{params} and \code{excl} are matched using regular expression format. This allows partial names to be used when specifying parameters of interest.
+#'
+#' @param mcmc.list Logical specifying whether to return an mcmc.list. If \code{TRUE}, an \code{mcmc.list} object is returned, rather than a matrix.
 #'
 #' @section Details:
-#' Function returns matrix with one chain per column for specified parameters. Multiple input chains for each parameter are combined to one posterior chain.
+#' Function returns matrix with one chain per column for specified parameters. Multiple input chains for each parameter are combined to one posterior chain. Parameters are arranged in columns alphabetically.
 #'
 #' \code{object} argument can be a \code{stanfit} object (\code{rstan} package), an \code{mcmc.list} object (\code{coda} package), an \code{R2jags} model object (\code{R2jags} package), or a matrix containing MCMC chains (each column representing MCMC output for a single parameter, rows representing iterations in the chain). The function automatically detects the object type and proceeds accordingly.
 #'
@@ -30,7 +32,8 @@
 #' apply(ex2, 2, mean)
 #'
 #' #Just 'beta[1]', 'gamma[4]', and 'alpha[3]'
-#' ex3 <- MCMCchains(MCMC_data, params = c('beta[1]', 'gamma[4]', 'alpha[3]'), ISB = FALSE)
+#' #'params' takes regular expressions when ISB = FALSE, square brackets must be escaped with '\\'
+#' ex3 <- MCMCchains(MCMC_data, params = c('beta\\[1\\]', 'gamma\\[4\\]', 'alpha\\[3\\]'), ISB = FALSE)
 #' apply(ex3, 2, sd)
 #'
 #' @export
@@ -38,70 +41,89 @@
 MCMCchains <- function(object,
                      params = 'all',
                      excl = NULL,
-                     ISB = TRUE)
+                     ISB = TRUE,
+                     mcmc.list = FALSE)
 {
-    #NAME SORTING BLOCK
-    if(typeof(object) == 'S4')
+  if(length(class(object)) > 1)
+  {
+    #modified coda::as.mcmc (removing ordering of param names)
+    x <- object$BUGSoutput
+    mclist <- vector("list", x$n.chains)
+    mclis <- vector("list", x$n.chains)
+    strt <- x$n.burnin + 1
+    end <- x$n.iter
+    ord <- dimnames(x$sims.array)[[3]]
+    for (i in 1:x$n.chains)
     {
-      temp_in <- rstan::As.mcmc.list(object)
-      if(ISB == TRUE)
-      {
-        names <- vapply(strsplit(colnames(temp_in[[1]]),
-                                 split = "[", fixed = TRUE), `[`, 1, FUN.VALUE=character(1))
-      }else{
-        names <- colnames(temp_in[[1]])
-      }
+      tmp1 <- x$sims.array[, i, ord]
+      mclis[[i]] <- coda::mcmc(tmp1, start = strt, end = end, thin = x$n.thin)
     }
+    object <- coda::as.mcmc.list(mclis)
+    #end mod as.mcmc
+  }
+  if(coda::is.mcmc.list(object) != TRUE &
+     typeof(object) != 'double' &
+     class(object) != 'rjags' &
+     typeof(object) != 'S4')
+  {
+    stop('Invalid object type. Input must be stanfit object (rstan), mcmc.list object (coda),
+         rjags object (R2jags), or matrix with MCMC chains.')
+  }
 
-    if(coda::is.mcmc.list(object) == TRUE)
+  #NAME SORTING BLOCK
+  if(typeof(object) == 'S4')
+  {
+    temp_in <- rstan::As.mcmc.list(object)
+    if(ISB == TRUE)
     {
-      temp_in <- object
-      if(ISB == TRUE)
-      {
-        names <- vapply(strsplit(colnames(temp_in[[1]]),
-                                 split = "[", fixed = TRUE), `[`, 1, FUN.VALUE=character(1))
-      }else{
-        names <- colnames(temp_in[[1]])
-      }
+      names <- vapply(strsplit(colnames(temp_in[[1]]),
+                               split = "[", fixed = TRUE), `[`, 1, FUN.VALUE=character(1))
+    }else{
+      names <- colnames(temp_in[[1]])
     }
+  }
 
-    if(typeof(object) == 'double')
+  if(coda::is.mcmc.list(object) == TRUE)
+  {
+    temp_in <- object
+    if(ISB == TRUE)
     {
-      temp_in <- object
-      if(ISB == TRUE)
-      {
-        names <- vapply(strsplit(colnames(temp_in),
-                                 split = "[", fixed = TRUE), `[`, 1, FUN.VALUE=character(1))
-      }else{
-        names <- colnames(temp_in)
-      }
+      names <- vapply(strsplit(colnames(temp_in[[1]]),
+                               split = "[", fixed = TRUE), `[`, 1, FUN.VALUE=character(1))
+    }else{
+      names <- colnames(temp_in[[1]])
     }
+  }
 
-    if(class(object) == 'rjags')
+  if(typeof(object) == 'double')
+  {
+    temp_in <- object
+    if(ISB == TRUE)
     {
-      temp_in <- object$BUGSoutput$sims.matrix
-      if(ISB == TRUE)
-      {
-        names <- vapply(strsplit(rownames(object$BUGSoutput$summary),
-                                 split = "[", fixed = TRUE), `[`, 1, FUN.VALUE=character(1))
-      }else{
-        names <- rownames(object$BUGSoutput$summary)
-      }
+      names <- vapply(strsplit(colnames(temp_in),
+                               split = "[", fixed = TRUE), `[`, 1, FUN.VALUE=character(1))
+    }else{
+      names <- colnames(temp_in)
     }
+  }
+
+  if(class(object) == 'rjags')
+  {
+    temp_in <- object$BUGSoutput$sims.matrix
+    if(ISB == TRUE)
+    {
+      names <- vapply(strsplit(rownames(object$BUGSoutput$summary),
+                               split = "[", fixed = TRUE), `[`, 1, FUN.VALUE=character(1))
+    }else{
+      names <- rownames(object$BUGSoutput$summary)
+    }
+  }
 
   if(class(object[[1]]) == 'mcarray')
   {
     stop('Invalid object type. jags.samples objects not currently supported. Input must be stanfit object, mcmc.list object, rjags object, or matrix with MCMC chains.')
   }
 
-  if(coda::is.mcmc.list(object) != TRUE &
-     typeof(object) != 'double' &
-     typeof(object) != 'list' &
-     typeof(object) != 'S4')
-  {
-    stop('Invalid object type. Input must be stanfit object (rstan), mcmc.list object (coda),
-         rjags object (R2jags), or matrix with MCMC chains.')
-  }
 
   #INDEX BLOCK
   #exclusions
@@ -117,9 +139,13 @@ MCMCchains <- function(object,
         rm_ind <- c(rm_ind, which(names %in% n_excl[i]))
       }else{
         n_excl <- excl
-        rm_ind <- c(rm_ind, grep(n_excl[i], names, fixed = TRUE))
+        rm_ind <- c(rm_ind, grep(n_excl[i], names, fixed = FALSE))
       }
     }
+    if(length(rm_ind) < 1)
+    {
+      stop(paste0('"', excl, '"', ' not found in MCMC ouput.'))
+    }
     dups <- which(duplicated(rm_ind))
     if(length(dups) > 0)
     {
@@ -145,7 +171,7 @@ MCMCchains <- function(object,
       {
         get_ind <- which(names %in% params)
       }else{
-        get_ind <- grep(paste(params), names, fixed = TRUE)
+        get_ind <- grep(paste(params), names, fixed = FALSE)
       }
 
       if (length(get_ind) < 1)
@@ -177,7 +203,7 @@ MCMCchains <- function(object,
       {
         get_ind <- which(names %in% params[i])
       }else{
-        get_ind <- grep(paste(params[i]), names, fixed=TRUE)
+        get_ind <- grep(paste(params[i]), names, fixed=FALSE)
       }
 
       if (length(get_ind) < 1)
@@ -220,6 +246,7 @@ MCMCchains <- function(object,
   }
 
 
+  #PROCESSING BLOCK
   if(coda::is.mcmc.list(object) == TRUE)
   {
     if(length(f_ind) > 1)
@@ -227,21 +254,51 @@ MCMCchains <- function(object,
       dsort <- do.call(coda::mcmc.list, temp_in[,f_ind])
       OUT <- do.call('rbind', dsort)
     }else{
-      dsort <- do.call(coda::mcmc.list, temp_in[,c(f_ind, f_ind)])
-      OUT <- do.call('rbind', dsort)[,1]
+      dsort <- do.call(coda::mcmc.list, temp_in[,f_ind, drop = FALSE])
+      OUT <- as.matrix(do.call(coda::mcmc.list, temp_in[,f_ind, drop = FALSE]), ncol = 1)
     }
   }
 
   if(typeof(object) == 'double')
   {
-    OUT <- temp_in[,f_ind]
+    OUT <- temp_in[,f_ind, drop = FALSE]
+    if(mcmc.list == TRUE)
+    {
+      stop('Cannot produce mcmc.list output with matrix input')
+    }
   }
 
-  if(typeof(object) == 'list' & coda::is.mcmc.list(object) == FALSE)
+  if(class(object) == 'rjags')
   {
-    OUT <- temp_in[,f_ind]
+    OUT <- temp_in[,f_ind, drop = FALSE]
+    if(mcmc.list == TRUE)
+    {
+      #modified coda::as.mcmc (removing ordering of param names)
+      x <- object$BUGSoutput
+      mclist <- vector("list", x$n.chains)
+      mclis <- vector("list", x$n.chains)
+      strt <- x$n.burnin + 1
+      end <- x$n.iter
+      ord <- dimnames(x$sims.array)[[3]]
+      for (i in 1:x$n.chains)
+      {
+        tmp1 <- x$sims.array[, i, ord]
+        mclis[[i]] <- coda::mcmc(tmp1, start = strt, end = end, thin = x$n.thin)
+      }
+      temp2 <- coda::as.mcmc.list(mclis)
+      #end mod as.mcmc
+      dsort <- do.call(coda::mcmc.list, temp2[,f_ind, drop = FALSE])
+    }
   }
 
-  return(OUT)
+  if(mcmc.list == FALSE)
+  {
+    return(OUT)
+  }
+
+  if(mcmc.list == TRUE)
+  {
+    return(dsort)
+  }
 }
 
