@@ -10,7 +10,7 @@
 #'
 #' @param excl Character string (or vector of character strings) denoting parameters to exclude. Used in conjunction with \code{params} argument to select parameters of interest.
 #'
-#' @param ISB Ignore Square Brackets (ISB). Logical specifying whether square brackets should be ignored in the \code{params} and \code{excl} arguments. If \code{FALSE}, square brackets are ignored - input from \code{params} and \code{excl} are otherwise matched exactly. If \code{TRUE}, square brackets are not ignored - input from \code{params} and \code{excl} are matched using grep, allowing partial names to be used when specifying parameters of interest.
+#' @param ISB Ignore Square Brackets (ISB). Logical specifying whether square brackets should be ignored in the \code{params} and \code{excl} arguments. If \code{TRUE}, square brackets are ignored - input from \code{params} and \code{excl} are otherwise matched exactly. If \code{FALSE}, square brackets are not ignored - input from \code{params} and \code{excl} are matched using grep, which can take arguments in regular expression format. This allows partial names to be used when specifying parameters of interest.
 #'
 #' @param digits Number of digits to include for posterior summary. Values will be rounded to the specified number of digits (except for Rhat which is always rounded to 2 digits).
 #'
@@ -54,7 +54,8 @@
 #' MCMCsummary(MCMC_data, params = 'beta')
 #'
 #' #Just 'beta[1]', 'gamma[4]', and 'alpha[3]'
-#' MCMCsummary(MCMC_data, params = c('beta[1]', 'gamma[4]', 'alpha[3]'), ISB = FALSE)
+#' #'params' takes regular expressions when ISB = FALSE, square brackets must be escaped with '\\'
+#' MCMCsummary(MCMC_data, params = c('beta\\[1\\]', 'gamma\\[4\\]', 'alpha\\[3\\]'), ISB = FALSE)
 #'
 #' @export
 
@@ -69,283 +70,53 @@ MCMCsummary <- function(object,
                       func = NULL,
                       func_name = NULL)
 {
-  if(coda::is.mcmc.list(object) != TRUE &
-     typeof(object) != 'double' &
-     typeof(object) != 'list' &
-     typeof(object) != 'S4')
+  #SORTING BLOCK
+  if(typeof(object) == 'double')
   {
-    stop('Invalid object type. Input must be stanfit object, mcmc.list object, rjags object, or matrix with MCMC chains.')
-  }
-
-  #NAME SORTING BLOCK
-  #R2jags object
-  if(class(object) == 'rjags')
-  {
-    if(ISB == TRUE)
-    {
-      names <- vapply(strsplit(rownames(object$BUGSoutput$summary),
-                               split = "[", fixed = TRUE), `[`, 1, FUN.VALUE=character(1))
-    }else{
-      names <- rownames(object$BUGSoutput$summary)
-    }
-  }else {
-    #Stan object
-    if(typeof(object) == 'S4')
-    {
-      object2 <- rstan::As.mcmc.list(object)
-    } else {
-      object2 <- object
-    }
-    #MCMClist object
-    if(coda::is.mcmc.list(object2) == TRUE)
-    {
-      temp <- object2
-      if(ISB == TRUE)
-      {
-        names <- vapply(strsplit(colnames(temp[[1]]),
-                                 split = "[", fixed = TRUE), `[`, 1, FUN.VALUE=character(1))
-      }else{
-        names <- colnames(temp[[1]])
-      }
-    }
-    #matrix object
-    if(typeof(object2) == 'double')
-    {
-      temp <- object2
-      if(ISB == TRUE)
-      {
-        names <- vapply(strsplit(colnames(temp),
-                                 split = "[", fixed = TRUE), `[`, 1, FUN.VALUE=character(1))
-      }else{
-        names <- colnames(temp)
-      }
-    }
-    #jags.samples object (mcarray)
-    if(class(object2[[1]]) == 'mcarray')
-    {
-      stop('Invalid object type. jags.samples objects not currently supported. Input must be stanfit object, mcmc.list object, rjags object, or matrix with MCMC chains.')
-    }
-  }
-
-
-  #INDEX BLOCK
-  #exclusions
-  if(!is.null(excl))
-  {
-    rm_ind <- c()
-    for (i in 1:length(excl))
-    {
-      if(ISB == TRUE)
-      {
-        n_excl <- vapply(strsplit(excl,
-                                 split = "[", fixed = TRUE), `[`, 1, FUN.VALUE=character(1))
-        rm_ind <- c(rm_ind, which(names %in% n_excl[i]))
-      }else{
-        n_excl <- excl
-        rm_ind <- c(rm_ind, grep(n_excl[i], names, fixed = TRUE))
-      }
-    }
-    if(length(rm_ind) < 1)
-    {
-      stop(paste0('"', excl, '"', ' not found in MCMC ouput.'))
-    }
-    dups <- which(duplicated(rm_ind))
-    if(length(dups) > 0)
-    {
-      rm_ind2 <- rm_ind[-dups]
-    }else{
-      rm_ind2 <- rm_ind
-    }
-  }
-
-  #selections
-  if (length(params) == 1)
-  {
-    if (params == 'all')
-    {
-      if(is.null(excl))
-      {
-        f_ind <- 1:length(names)
-      }else{
-        f_ind <- (1:length(names))[-rm_ind2]
-      }
-    }else {
-      if(ISB == TRUE)
-      {
-        get_ind <- which(names %in% params)
-      }else{
-        get_ind <- grep(paste(params), names, fixed = TRUE)
-      }
-
-      if (length(get_ind) < 1)
-      {
-        stop(paste0('"', params, '"', ' not found in MCMC ouput.'))
-      }
-      if(!is.null(excl))
-      {
-        if(identical(get_ind, rm_ind2))
-        {
-          stop('No parameters selected.')
-        }
-        matched <- stats::na.omit(match(rm_ind2, get_ind))
-        if (length(matched) > 0)
-        {
-          f_ind <- get_ind[-matched]
-        }else {
-          f_ind <- get_ind
-        }
-      }else {
-        f_ind <- get_ind
-      }
-    }
-  }else {
-    grouped <- c()
-    for (i in 1:length(params))
-    {
-      if(ISB == TRUE)
-      {
-        get_ind <- which(names %in% params[i])
-      }else{
-        get_ind <- grep(paste(params[i]), names, fixed=TRUE)
-      }
-
-      if (length(get_ind) < 1)
-      {
-        stop(paste0('"', params[i], '"', ' not found in MCMC ouput.'))
-      }
-      grouped <- c(grouped, get_ind)
-    }
-    if(!is.null(excl))
-    {
-      if(identical(grouped, rm_ind2))
-      {
-        stop('No parameters selected.')
-      }
-      matched <- stats::na.omit(match(rm_ind2, grouped))
-      if (length(matched) > 0)
-      {
-        t_ind <- grouped[-matched]
-      } else{
-        t_ind <- grouped
-      }
-      to.rm <- which(duplicated(t_ind))
-      if(length(to.rm) > 0)
-      {
-        f_ind <- t_ind[-to.rm]
-      }else
-      {
-        f_ind <- t_ind
-      }
-    } else{
-      to.rm <- which(duplicated(grouped))
-      if(length(to.rm) > 0)
-      {
-        f_ind <- grouped[-to.rm]
-      }else
-      {
-        f_ind <- grouped
-      }
-    }
+    object2 <- MCMCchains(object, params, excl, ISB, mcmc.list = FALSE)
+  }else{
+    object2 <- MCMCchains(object, params, excl, ISB, mcmc.list = TRUE)
   }
 
 
   #PROCESSING BLOCK
-  if(typeof(object) == 'list' & coda::is.mcmc.list(object) == FALSE)
-  {
-    if(Rhat == TRUE)
-    {
-      rh <- round(object$BUGSoutput$summary[,8], digits = 2)
-      x <- cbind(round(object$BUGSoutput$summary[,c(1, 2, 3, 5, 7)], digits = digits), rh)
-      colnames(x)[6] <- 'Rhat'
-    }else{
-      x <- cbind(round(object$BUGSoutput$summary[,c(1, 2, 3, 5, 7)], digits = digits))
-    }
-
-    if(n.eff == TRUE)
-    {
-      nf <- round(object$BUGSoutput$summary[,9], digits = 0)
-      x2 <- cbind(x, nf)
-      colnames(x2)[ncol(x2)] <- 'n.eff'
-    }else{
-      x2 <- x
-    }
-
-    x2 <- x2[f_ind, , drop = FALSE]
-
-    if(!is.null(func))
-    {
-      if(length(f_ind) > 1)
-      {
-        ch_bind <- object$BUGSoutput$sims.matrix[,f_ind]
-      }else{
-        ch_bind <- as.matrix(object$BUGSoutput$sims.matrix[,f_ind], ncol = 1)
-      }
-
-      tmp <- round(apply(ch_bind, 2, func), digits = digits)
-
-      if(!is.null(dim(tmp)) & NROW(tmp) > 1)
-      {
-        x3 <- x2
-        for (i in 1:NROW(tmp))
-        {
-          x3 <- cbind(x3, tmp[i,])
-          if (!is.null(func_name))
-          {
-            if (length(func_name) != NROW(tmp))
-            {
-              stop('length(func_name) must equal number of func outputs')
-            }
-            colnames(x3)[ncol(x2) + i] <- func_name[i]
-          }else{
-            colnames(x3)[ncol(x2) + i] <- 'func'
-          }
-        }
-      }else{
-        x3 <- cbind(x2, tmp)
-        if (!is.null(func_name))
-        {
-          if (length(func_name) > 1)
-          {
-            stop('length(func_name) must equal number of func outputs')
-          }
-          colnames(x3)[ncol(x3)] <- func_name
-        }else{
-          colnames(x3)[ncol(x3)] <- 'func'
-        }
-      }
-    }else{
-      x3 <- x2
-    }
-
-    mcmc_summary <- x3
-
-  }else{
-
     if(coda::is.mcmc.list(object2) == TRUE)
     {
-      if(length(f_ind) > 1)
+      np <- NCOL(object2[[1]])
+
+      if(np > 1)
       {
-        dsort <- do.call(coda::mcmc.list, temp[,f_ind])
-        ch_bind <- do.call('rbind', dsort)
+        ch_bind <- do.call('rbind', object2)
       }else{
-        dsort <- do.call(coda::mcmc.list, temp[,f_ind])
-        ch_bind <- as.matrix(do.call(coda::mcmc.list, temp[,f_ind]), ncol = 1)
+        ch_bind <- as.matrix(object2)
       }
 
       bind_mn <- round(apply(ch_bind, 2, mean), digits = digits)
       bind_sd <- round(apply(ch_bind, 2, stats::sd), digits = digits)
       bind_LCI <- round(apply(ch_bind, 2, stats::quantile, probs= 0.025), digits = digits)
-      bind_med <- round(apply(ch_bind,2, stats::median), digits = digits)
+      bind_med <- round(apply(ch_bind, 2, stats::median), digits = digits)
       bind_UCI <- round(apply(ch_bind, 2, stats::quantile, probs= 0.975), digits = digits)
 
       if(Rhat == TRUE)
       {
-        if(length(dsort) > 1)
+        if(length(object2) > 1)
         {
-          r_hat <- round(coda::gelman.diag(dsort, multivariate = FALSE)$psrf[,1], digits = 2)
+          #If > 750 params use loop to calc Rhat
+          if(NCOL(object2[[1]]) > 750)
+          {
+            r_hat <- c(rep(NA, NCOL(object2[[1]])))
+            for (v in 1:length(r_hat))
+            {
+              r_hat[v] <- round(coda::gelman.diag(object2[,v])$psrf[,1], digits = 2)
+            }
+
+          }else{
+            r_hat <- round(coda::gelman.diag(object2, multivariate = FALSE)$psrf[,1], digits = 2)
+          }
+
         }else{
           warning('Rhat statistic cannot be calculated with one chain. NAs inserted.')
-          r_hat <- rep(NA, NCOL(dsort))
+          r_hat <- rep(NA, NCOL(object2))
         }
         x <- cbind(bind_mn, bind_sd, bind_LCI, bind_med, bind_UCI, r_hat)
         colnames(x) <- c('mean', 'sd', '2.5%','50%','97.5%', 'Rhat')
@@ -356,7 +127,7 @@ MCMCsummary <- function(object,
 
       if(n.eff == TRUE)
       {
-        bind_neff <- round(coda::effectiveSize(dsort), digits = 0)
+        bind_neff <- round(coda::effectiveSize(object2), digits = 0)
         x2 <- cbind(x, bind_neff)
         colnames(x2)[ncol(x2)] <- 'n.eff'
       }else{
@@ -400,31 +171,25 @@ MCMCsummary <- function(object,
       }else{
         x3 <- x2
       }
-
-      #needed when length(f_ind) == 1
-      rownames(x3) <- colnames(temp[[1]])[f_ind]
       mcmc_summary <- x3
     }
 
+
     if(typeof(object2) == 'double')
     {
-      if(length(f_ind) > 1)
-      {
-        dsort <- temp[,f_ind]
-      }else{
-        dsort <- as.matrix(temp[,f_ind], ncol = 1)
-      }
+      np <- NCOL(object2)
+      ch_bind <- object2
 
-      bind_mn <- round(apply(dsort, 2, mean), digits = digits)
-      bind_sd <- round(apply(dsort, 2, stats::sd), digits = digits)
-      bind_LCI <- round(apply(dsort, 2, stats::quantile, probs= 0.025), digits = digits)
-      bind_med <- round(apply(dsort,2, stats::median), digits = digits)
-      bind_UCI <- round(apply(dsort, 2, stats::quantile, probs= 0.975), digits = digits)
+      bind_mn <- round(apply(ch_bind, 2, mean), digits = digits)
+      bind_sd <- round(apply(ch_bind, 2, stats::sd), digits = digits)
+      bind_LCI <- round(apply(ch_bind, 2, stats::quantile, probs= 0.025), digits = digits)
+      bind_med <- round(apply(ch_bind, 2, stats::median), digits = digits)
+      bind_UCI <- round(apply(ch_bind, 2, stats::quantile, probs= 0.975), digits = digits)
 
       if(Rhat == TRUE)
       {
         warning('Rhat statistic cannot be calculated without individual chains. NAs inserted.')
-        r_hat <- rep(NA, NCOL(dsort))
+        r_hat <- rep(NA, np)
         x <- cbind(bind_mn, bind_sd, bind_LCI, bind_med, bind_UCI, r_hat)
         colnames(x) <- c('mean', 'sd', '2.5%','50%','97.5%', 'Rhat')
       }else{
@@ -435,7 +200,7 @@ MCMCsummary <- function(object,
       if(n.eff == TRUE)
       {
         warning('Number of effective samples cannot be calculated without individual chains. NAs inserted.')
-        bind_neff <- rep(NA, NCOL(dsort))
+        bind_neff <- rep(NA, np)
         x2 <- cbind(x, bind_neff)
         colnames(x2)[ncol(x2)] <- 'n.eff'
       }else{
@@ -444,7 +209,7 @@ MCMCsummary <- function(object,
 
       if(!is.null(func))
       {
-        tmp <- round(apply(dsort, 2, func), digits = digits)
+        tmp <- round(apply(ch_bind, 2, func), digits = digits)
 
         if(!is.null(dim(tmp)) & NROW(tmp) > 1)
         {
@@ -479,11 +244,7 @@ MCMCsummary <- function(object,
       }else{
         x3 <- x2
       }
-
-      #needed when length(f_ind) == 1
-      rownames(x3) <- colnames(temp)[f_ind]
       mcmc_summary <- x3
     }
-  }
   return(mcmc_summary)
 }
